<?php

namespace App\Service;

use App\Model\Employee;
use App\Model\Product;
use App\Model\ProductCategory;
use App\Model\ProductInfo;
use App\Model\ProductInventory;
use App\Model\ProductRange;
use Illuminate\Support\Facades\DB;

class ProductService extends Service
{
    public function productCategoryEdit($data,$user){
        list($status,$msg) = $this->productCategoryRule($data,false);
        if(!$status) return [$status,$msg];

        $update = $msg['data'][0];

        $model = new ProductCategory();
        $model->where('id',$data['id'])->update($update);

        return [true,''];
    }

    public function productCategoryAdd($data,$user){
        list($status,$msg) = $this->productCategoryRule($data);
        if(!$status) return [$status,$msg];

        ProductCategory::insert($msg['data']);

        return [true,''];
    }

    public function productCategoryDel($data){
        if($this->isEmpty($data,'id')) return [false,'请选择数据！'];

        $bool = Product::where('del_time',0)
            ->where('product_category_id',$data['id'])
            ->exists();
        if($bool) return [false,'产品分类下已添加产品，操作失败'];

        try {
            DB::beginTransaction();

            ProductCategory::where('id',$data['id'])->update([
                'del_time' => time()
            ]);

            DB::commit();
        }catch (\Exception $exception){
            DB::rollBack();
            return [false,$exception->getMessage()];
        }

        return [true,''];
    }

    public function productCategoryList($data,$user){
        $model = ProductCategory::where('del_time',0)
            ->select('title','id','parent_id')
            ->orderby('id','desc');
        if(! empty($data['title'])) $model->where('title', 'LIKE', '%'.$data['title'].'%');

        $list = $model->get()->toArray();
        if(! empty($list)) {
            $list = $this->makeTree(0,$list);
            $list = $this->set_sort_circle($list);
        }

        return [true, $list];
    }

    public function productCategoryRule($data, $is_add = true){
        if($this->isEmpty($data,'data')) return [false,'数据不能为空！'];

        $title = array_column($data['data'],'title');
        $title = array_map(function($val) {
            return $val !== null ? $val : 0;
        }, $title);
        $title_count = array_count_values($title);
        foreach ($title as $value){
            if(empty($value)) return [false,'名称不能为空！'];
            if($title_count[$value] > 1) return [false,'名称不能重复'];
        }

        foreach ($data['data'] as $key => $value){
            $data['data'][$key]['upd_time'] = time();

            if($is_add){
                $parent_id = 0;
                if(! empty($value['parent_id'])) {
                    $bool = Product::where('del_time',0)
                        ->where('product_category_id',$value['parent_id'])
                        ->exists();
                    if($bool) {
                        $title = ProductCategory::where('id',$value['parent_id'])->select('title')->value();
                        return [false,'产品分类:'. $title .'下已添加产品，不允许添加子分类'];
                    }
                    $parent_id = $value['parent_id'];
                }

                $data['data'][$key]['parent_id'] = $parent_id;
                $data['data'][$key]['crt_time'] = time();

                $bool = ProductCategory::where('title',$value['title'])
                    ->where('del_time',0)
                    ->exists();
            }else{
                if($this->isEmpty($data,'id')) return [false,'id不能为空！'];

                $bool = ProductCategory::where('title',$value['title'])
                    ->where('id','<>',$data['id'])
                    ->where('del_time',0)
                    ->exists();
            }
            if($bool) return [false,'分类名称不能重复'];
        }

        return [true, $data];
    }

    public function productEdit($data,$user){
        list($status,$msg) = $this->productRule($data,false);
        if(!$status) return [$status,$msg];

        try {
            DB::beginTransaction();

            $model = Product::where('id',$data['id'])->first();
            $model->product_category_id = $data['product_category_id'] ?? 0;
            $model->title = $data['title'];
            $model->code = $data['code'] ?? '';
            $model->size = $data['size'] ?? '';
            $model->unit = $data['unit'] ?? '';
            $model->bar_code = $data['bar_code'] ?? '';
            $model->retail_price = $data['retail_price'] ?? 0;
            $model->mark = $data['mark'] ?? '';
            $model->state = $data['state'] ?? 0;
            $model->save();

            $time = time();

            ProductInfo::where('del_time',0)
                ->where('product_id',$data['id'])
                ->update(['del_time' => $time]);

            if(! empty($data['img'])){
                $insert = [];
                foreach ($data['img'] as $value){
                    $insert[] = [
                        'product_id' => $model->id,
                        'file' => $value,
                        'type' => ProductInfo::type_one,
                        'crt_time' => $time,
                    ];
                }
                ProductInfo::insert($insert);
            }

            if(! empty($data['file'])){
                $insert = [];
                foreach ($data['file'] as $value){
                    $insert[] = [
                        'product_id' => $model->id,
                        'file' => $value,
                        'type' => ProductInfo::type_two,
                        'crt_time' => $time,
                    ];
                }
                ProductInfo::insert($insert);
            }

            ProductRange::where('del_time',0)
                ->where('product_id',$data['id'])
                ->update(['del_time' => $time]);

            if(! empty($data['depart'])){
                $insert = [];
                foreach ($data['depart'] as $value){
                    $insert[] = [
                        'product_id' => $model->id,
                        'file' => $value,
                        'type' => ProductRange::type_one,
                        'crt_time' => $time,
                    ];
                }
                ProductRange::insert($insert);
            }

            if(! empty($data['employee'])){
                $insert = [];
                foreach ($data['employee'] as $value){
                    $insert[] = [
                        'product_id' => $model->id,
                        'file' => $value,
                        'type' => ProductRange::type_two,
                        'crt_time' => $time,
                    ];
                }
                ProductRange::insert($insert);
            }

            DB::commit();
        }catch (\Exception $exception){
            DB::rollBack();
            return [false,$exception->getMessage()];
        }

        return [true,''];
    }

    public function productAdd($data,$user){
        list($status,$msg) = $this->productRule($data);
        if(!$status) return [$status,$msg];

        try {
            DB::beginTransaction();

            $model = new Product();
            $model->product_category_id = $data['product_category_id'] ?? 0;
            $model->title = $data['title'];
            $model->code = $data['code'] ?? '';
            $model->size = $data['size'] ?? '';
            $model->unit = $data['unit'] ?? '';
            $model->bar_code = $data['bar_code'] ?? '';
            $model->retail_price = $data['retail_price'] ?? 0;
            $model->mark = $data['mark'] ?? '';
            $model->state = $data['state'] ?? 0;
            $model->crt_id = $user['id'];
            $model->save();

            $time = time();

            if(! empty($data['img'])){
                $insert = [];
                foreach ($data['img'] as $value){
                    $insert[] = [
                        'product_id' => $model->id,
                        'file' => $value,
                        'type' => ProductInfo::type_one,
                        'crt_time' => $time,
                    ];
                }
                ProductInfo::insert($insert);
            }

            if(! empty($data['file'])){
                $insert = [];
                foreach ($data['file'] as $value){
                    $insert[] = [
                        'product_id' => $model->id,
                        'file' => $value,
                        'type' => ProductInfo::type_two,
                        'crt_time' => $time,
                    ];
                }
                ProductInfo::insert($insert);
            }

            if(! empty($data['depart'])){
                $insert = [];
                foreach ($data['depart'] as $value){
                    $insert[] = [
                        'product_id' => $model->id,
                        'file' => $value,
                        'type' => ProductRange::type_one,
                        'crt_time' => $time,
                    ];
                }
                ProductRange::insert($insert);
            }

            if(! empty($data['employee'])){
                $insert = [];
                foreach ($data['employee'] as $value){
                    $insert[] = [
                        'product_id' => $model->id,
                        'file' => $value,
                        'type' => ProductRange::type_two,
                        'crt_time' => $time,
                    ];
                }
                ProductRange::insert($insert);
            }

            DB::commit();
        }catch (\Exception $exception){
            DB::rollBack();
            return [false,$exception->getMessage()];
        }

        return [true,''];
    }

    public function productDel($data){
        if($this->isEmpty($data,'id')) return [false,'请选择数据！'];

        try {
            DB::beginTransaction();
            $time = time();

            Product::where('id',$data['id'])->update(['del_time' => $time]);

            ProductInfo::where('del_time',0)
                ->where('product_id',$data['id'])
                ->update(['del_time' => $time]);

            ProductRange::where('del_time',0)
                ->where('product_id',$data['id'])
                ->update(['del_time' => $time]);

            DB::commit();
        }catch (\Exception $exception){
            DB::rollBack();
            return [false,$exception->getMessage()];
        }

        return [true,''];
    }

    public function productDetail($data,$user){
        if($this->isEmpty($data,'id')) return [false,'请选择数据！'];

        $customer = Product::where('del_time',0)
            ->where('id',$data['id'])
            ->first();
        if(empty($customer)) return [false,'产品不存在或已被删除'];
        $customer = $customer->toArray();
        //库存
        $customer['product_inventory'] = [];
        $depart_id = array_column($user['rule_depart'],'depart_id');
        $inventory = $this->getProductInventory([$data['id']], $depart_id);
        if(! empty($inventory)) $customer['product_inventory'] = $inventory;

        $customer['img'] = $customer['file'] = $customer['depart'] = $customer['employee'] = [];
        $customer_info = ProductInfo::where('del_time',0)
            ->where('product_id',$customer['id'])
            ->select('id','product_id','file','type')
            ->get()->toArray();
        foreach ($customer_info as $value){
            if($value['type'] == ProductInfo::type_one){
                $customer['img'][] = [
                    $value['file']
                ];
            }elseif ($value['type'] == ProductInfo::type_two){
                $customer['file'][] = [
                    $value['file']
                ];
            }
        }

        $customer_range = ProductRange::where('del_time',0)
            ->where('product_id',$customer['id'])
            ->select('id','product_id','depart_id','type','employee_id')
            ->get()->toArray();
        foreach ($customer_range as $value){
            if($value['type'] == ProductRange::type_one){
                $customer['depart'][] = [
                    $value['depart_id']
                ];
            }elseif ($value['type'] == ProductRange::type_two){
                $customer['employee'][] = [
                    $value['employee_id']
                ];
            }
        }

        return [true, $customer];
    }

    public function productList($data,$user){
        $model = Product::where('del_time',0)
            ->select('title','id','product_category_id','code','size','unit','bar_code','retail_price','state','crt_id','crt_time','mark')
            ->orderby('id', 'desc');

        if(! $user['is_main']) {
            $user_id = $user['id'];
            $depart_id = array_column($user['rule_depart'],'depart_id');
            $product_id = ProductRange::where('del_time',0)
                ->where(function ($query) use($user_id, $depart_id) {
                    $query->where('employee_id',$user_id)
                        ->orWhereIn('depart_id', $depart_id);
                })->select('product_id')->get()
                ->toArray();
            $model->whereIn('id',array_column($product_id,'product_id'));
        }
        if(! empty($data['title'])) $model->where('title', 'LIKE', '%'.$data['title'].'%');
        if(isset($data['state'])) $model->where('state', $data['state']);

        $list = $this->limit($model,'',$data);
        $list = $this->fillData($list);

        return [true, $list];
    }

    public function productRule($data, $is_add = true){
        if(empty($data['title'])) return [false,'产品名称不能为空'];
        if(empty($data['product_category_id'])) return [false,'产品分类不能为空'];
        if(empty($data['code'])) return [false,'产品编码不能为空'];
        if(! empty($data['retail_price'])){
            $res = $this->checkNumber($data['retail_price']);
            if(! $res) return [false,'零售价请输入不超过两位小数的数值'];
        }

        if($is_add){
            $bool = Product::whereRaw("(binary code = '{$data['code']}' OR title = '{$data['title']}')")
                ->where('del_time',0)
                ->exists();
        }else{
            if(empty($data['id'])) return [false,'ID不能为空'];
            $bool = Product::whereRaw("(binary code = '{$data['code']}' OR title = '{$data['title']}')")
                ->where('id','<>',$data['id'])
                ->where('del_time',0)
                ->exists();
        }
        if($bool) return [false,'产品名称或编码不能重复'];

        return [true, $data];
    }

    public function fillData($data){
        if(empty($data['data'])) return $data;

        $emp = Employee::whereIn('id',array_unique(array_column($data['data'],'crt_id')))
            ->pluck('emp_name','id')
            ->toArray();
        $category = ProductCategory::whereIn('id',array_unique(array_column($data['data'],'product_category_id')))
            ->pluck('title','id')
            ->toArray();

        foreach ($data['data'] as $key => $value){
            $data['data'][$key]['crt_time'] = $value['crt_time'] ? date('Y-m-d H:i:s',$value['crt_time']) : '';
            $data['data'][$key]['crt_name'] = $emp[$value['crt_id']] ?? '';
            $data['data'][$key]['product_category_name'] = $category[$value['product_category_id']] ?? '';
            $data['data'][$key]['state_name'] = Product::$state[$value['state']] ?? '';
        }

        return $data;
    }

    public function getProductInventory($product_id = [],$depart_id = []){
        if(empty($product_id)) return [];

        $result = ProductInventory::whereIn('product_id',$product_id)
            ->when(! empty($depart_id), function ($query) use ($depart_id) {
                return $query->whereIn('depart_id', $depart_id);
            })
            ->select('id','product_id','number','depart_id','lock_number')
            ->get()
            ->toArray();

        return $result;
    }
}
